#!/usr/bin/env python3

import pytest
from copy import copy
from collections.abc import Mapping
from contextlib2 import nullcontext
from functools import partial

class Namespace(dict):
    """\
    Evaluate and/or execute snippets of python code, with powerful control over the 
    names available to those snippets.

    .. note::
    
        It is conventional to:

        - Name your namespaces: ``with_{short description}``

        - Create any namespaces you will need in a single helper file (e.g.  
          ``param_helpers.py``) to be imported in test scripts as necessary.

    Examples:

        The first step when using a namespace specify which names it should 
        include.   This can be done using...

        ...strings (which will be `python:exec`'d):
            
            >>> with_math = Namespace('import math')
            >>> with_math.eval('math.sqrt(4)')
            2.0

        ...dictionaries:

            >>> import math
            >>> with_math = Namespace(globals())
            >>> with_math.eval('math.sqrt(4)')
            2.0

        ...keyword arguments:

            >>> with_math = Namespace(math=math)
            >>> with_math.eval('math.sqrt(4)')
            2.0

        ...other namespaces (`Namespace.use` works just like the constructor):

            >>> with_np = with_math.copy().use('import numpy as np')
            >>> with_np.eval('np.arange(3) * math.sqrt(4)')
            array([0., 2., 4.])

        ...the `Namespace.all` method:

            >>> with_math = Namespace().all(math)
            >>> with_math.eval('sqrt(4)')
            2.0

        Once you have an initialized a namespace, you can use it to...

        ...evaluate expressions:

            >>> with_math.eval('sqrt(4)')
            2.0
            
        ...execute blocks of code:

            >>> ns = with_math.exec('''
            ... a = sqrt(4)
            ... b = sqrt(9)
            ... ''')
            >>> ns['a']
            2.0
            >>> ns['b']
            3.0
            >>> ns.eval('a + b')
            5.0

        ...make error-detecting context managers:

            >>> class MyError(Exception):
            ...     pass
            ...
            >>> with_err = Namespace(MyError=MyError)
            >>> err_cm = with_err.error({'type': 'MyError', 'message': r'\\d+'})
            >>> with err_cm:
            ...    raise MyError('404')

    If you plan to use a namespace as part of a schema, you probably want 
    `voluptuous.Namespace` instead of this class.
    """

    def __init__(self, *args, **kwargs):
        """
        See `use`.
        """
        super().__init__()
        self.use(*args, **kwargs)

    def __repr__(self):
        return f'{self.__class__.__name__}({super().__repr__()})'

    def use(self, *args, **kwargs):
        """
        Add the given names to the namespace.

        Arguments:
            args (str,dict):
                If string: The string will be executed as python code and any 
                names that are defined by that code will be added to the 
                namespace.  Any names previously added to the namespace will be 
                available to the code.

                If dict: The items in the dictionary will the directly added to 
                the namespace.  All of the dictionary keys must be strings.

                The *args* are processed in order, so later *args* may 
                overwrite earlier *args*.

            kwargs:
                Each key/value pair will be added to the namespace.  The 
                *kwargs* are processed after the *args*, so if the same key is 
                defined by both, the *kwargs* definition will be used.

        Returns:
            The namespace itself is returned, to support method chaining.
        """
        for arg in args:
            if isinstance(arg, Mapping):
                self.update(arg)
            else:
                exec(arg, self)

        self.update(kwargs)
        return self

    def all(self, module):
        """
        Add all names from the given module to the namespace.

        This method follows the same rules as ``from <module> import *``:

        - If the given module defines ``__all__``, only those names will be 
          used.
        - Otherwise, all names without leading underscores will be used.

        Returns:
            The namespace itself is returned, to support method chaining.
        """
        try:
            keys = module.__all__
        except AttributeError:
            keys = (k for k in module.__dict__ if not k.startswith('_'))

        self.update({
                k: module.__dict__[k]
                for k in keys
        })
        return self

    def copy(self):
        """
        Return a shallow copy of this namespace.  This is meant to help support 
        method chaining.
        """
        return copy(self)

    def eval(self, *src, eval_keys=False):
        """
        Evaluate the given expression within this namespace.
        this object.

        Arguments:
            src (str,list,dict):
                The expression (or expressions) to evaluate.  Strings are 
                directly evaluated.  List items and dict values are recursively 
                evaluated.  Dict keys are not evaluated unless *eval_keys* is 
                true.  This allows you to switch freely between structured text 
                and python syntax, depending on which makes most sense for each 
                particular input.

            eval_keys (bool):
                If true, evaluate dictionary keys.  Disabled by default because 
                most dictionary keys are strings, and it's annoying to have to 
                quote them.

        Returns:
            Any: The result of evaluating the given expressions.  
        """
        src = src[0] if len(src) == 1 else list(src)
        recurse = partial(self.eval, eval_keys=eval_keys)

        if type(src) is list:
            return [recurse(x) for x in src]
        elif type(src) is dict:
            f = recurse if eval_keys else lambda x: x
            return {f(k): recurse(v) for k, v in src.items()}
        else:
            return eval(src, self)

    def exec(self, src):
        """
        Execute the given python snippet within this namespace.

        Arguments:
            src (str): A snippet of python code to execute.

        Returns:
            Namespace: A new namespace containing all of the variables defined 
            in the snippet.

        """
        globals = self.copy()
        exec(src, globals)
        return globals

    def exec_and_lookup(self, key):
        """
        Execute a python snippet and return a specific variable.

        Arguments:
            key (str, collections.abc.Callable):
                If string: the name of the variable to return.  
                
                If callable: The given function will be passed a dictionary 
                containing all the names defined in the given snippet.  
                Whatever that function returns will be passed on to the caller.

        Returns:
            collections.abc.Callable:
                A closure that takes a snippet of python code, executes it, and 
                returns the value indicated by the given key.  While it may 
                seem counter-intuitive for this method to return a 
                snippet-executing function instead of simply executing snippets 
                itself, this API is designed to be used when defining schemas.  
                Be aware that having schemas execute large blocks of code is 
                usually a :ref:`bad idea <schema-be-careful>`, though.
        """

        def do_exec(src):
            globals = self.exec(src)

            if callable(key):
                return key(globals)
            else:
                return globals[key]

        return do_exec

    def error(self, params):
        """\
        Create a context manager that will check that a particular error is 
        raised.

        Arguments:
            params (str, dict):
                This argument specifies what exception to expect (and 
                optionally how to check its message).

                If string: A string that should evaluate to an exception type.  
                This can also be ``"none"`` to specify that the returned 
                context manager should not expect any exception to be raised.

                If dict: The following keys are understood:

                - "type" (required): A string that should evaluate to an 
                  exception type.  The context manager will require that an 
                  exception of that type is raised.

                - "message" (optional): A string or list of strings that should 
                  appear in the error message.  Each string is interpreted as a 
                  regular expression, in the same manner as the *match* 
                  argument to `pytest.raises`.

                Note that everything is expected to be strings, because this 
                method is meant to help with parsing exception information from 
                a text file, e.g. in the NestedText_ format.

        Returns:
            A context manager that can be used to check if the kind of 
            exception specified by *params* was raised.

        Examples:
            Using a built-in exception (so no need to specify globals) and not 
            checking the error message::

                >>> p = {'type': 'ZeroDivisionError'}
                >>> with Namespace().error(p):
                ...    1/0

            Using a custom exception::

                >>> class MyError(Exception):
                ...     pass
                ...
                >>> with_err = Namespace(MyError=MyError)
                >>> p = {'type': 'MyError', 'message': r'\\d+'}
                >>> with with_err.error(p):
                ...    raise MyError('404')

        Details:
            The returned context manager is re-entrant, which makes it possible 
            to stack :deco:`parametrize_from_file` invocations that make use of 
            method (e.g. via the *schema* argument).
        """
        if params == 'none':
            return nullcontext()

        if isinstance(params, str):
            err_type = self.eval(params)
            err_messages = []
        else:
            err_type = self.eval(params['type'])
            err_messages = params.get('message', [])
            if not isinstance(err_messages, list):
                err_messages = [err_messages]

        # Normally I'd use `@contextmanager` to make a context manager like 
        # this, but generator-based context managers cannot be reused.  This is 
        # a problem for tests, because if a test using this context manager is 
        # parametrized, the same context manager instance will need to be 
        # reused multiple times.  The only way to support this is to implement 
        # the context manager from scratch.

        class ExpectError:

            def __repr__(self):
                return f'<{self.__class__.__name__} type={err_type!r} messages={err_messages!r}>'

            def __enter__(self):
                self.raises = pytest.raises(err_type)
                self.err = self.raises.__enter__()

            def __exit__(self, *args):
                if self.raises.__exit__(*args):
                    for msg in err_messages:
                        self.err.match(msg)
                    return True

        return ExpectError()




#!/usr/bin/env python3

"""
Parametrize test functions with values read from config files.
"""

from .parametrize import parametrize_from_file, load_parameters
from .namespace import Namespace
from .loaders import add_loader, drop_loader
from .errors import ConfigError

__version__ = '0.5.1'

for obj in [parametrize_from_file, Namespace, ConfigError, add_loader, drop_loader, load_parameters]:
    obj.__module__ = 'parametrize_from_file'

# Hack to make the module directly usable as a decorator.  Only works for 
# python 3.5 or higher.  See this Stack Overflow post:
# https://stackoverflow.com/questions/1060796/callable-modules

import sys
import functools

class CallableModule(sys.modules[__name__].__class__):

    @functools.wraps(parametrize_from_file)
    def __call__(self, *args, **kwargs):
        return parametrize_from_file(*args, **kwargs)

sys.modules[__name__].__class__ = CallableModule
del sys, functools, CallableModule



#!/usr/bin/env python3

from .namespace import Namespace as BaseNamespace
from voluptuous import Schema, Optional, Invalid, Or, All
from unittest.mock import MagicMock
from tidyexc import only_raise

class Namespace(BaseNamespace):
    """
    A variant of the |NS| class, with tweaks to work better with voluptuous_.
    """

    @only_raise(Invalid)
    def eval(self, *src, **kwargs):
        """
        |NS_eval|, with tweaks to work better with voluptuous_.

        Specifically, this function is guaranteed to raise `voluptuous.Invalid 
        <voluptuous>`_ if for any reason the given expression(s) can not be 
        successfully evaluated.  This ensures that voluptuous_ will be able to 
        produce good error messages.
        """
        return super().eval(*src, **kwargs)

    @only_raise(Invalid)
    def exec(self, src):
        """
        |NS_exec|, with tweaks to work better with voluptuous_.

        See `eval` for details.
        """
        return super().exec(src)

    def exec_and_lookup(self, key):
        """
        |NS_exec_lookup|, with tweaks to work better with voluptuous_.

        See `eval` for details.
        """
        f = super().exec_and_lookup(key)
        return only_raise(Invalid)(f)

    @only_raise(Invalid)
    def error(self, params):
        """
        |NS_error|, with tweaks to work better with voluptuous_.

        See `eval` for details.
        """
        return super().error(params)

    def error_or(self, expected):
        """\
        Build a schema that will expect to be given either an error or the 
        specified set of expected values.

        Arguments:
            expected (dict):
                Any test parameters that must be specified when an error is 
                *not* expected.  This dictionary has can be almost anything 
                that you would pass to `voluptuous.Schema <voluptuous>`_, 
                except that the keys must be either strings or 
                `voluptuous.Optional <voluptuous>`_.

        Returns:
            dict:
                A voluptuous schema that can accept either a single parameter 
                named *error*, or whatever parameters are specified by the 
                *expected* argument.  When this schema is evaluated, the 
                results will depend on which parameter(s) are given:

                If the *error* parameter is given, it will be processed by 
                |NS_error|.  That means: (i) it should match the format 
                expected by that method and (ii) it will be converted into a 
                context manager that can be used to detect whether the expected 
                exception was in fact raised.  All of the keys in the 
                *expected* schema passed to this function will also be present 
                in the resulting output, and will be set to |MagicMock| objects.  
                This ensures that any comparisons involving those keys will 
                succeed, to help prevent the expected exception from being 
                preempted by some missing value.

                If the *error* parameter is not given, the resulting output 
                will contain an *error* key set to a no-op context manager.  
                All of the keys in the *expected* schema will be evaluated as 
                usual.

        Example:
            This example is somewhat contrived, but it shows how to easily test a 
            function that raises a custom exception for certain inputs.
            
               >>> from voluptuous import Schema
               >>> from parametrize_from_file.voluptuous import Namespace

               >>> class DontGreet(Exception):
               ...     pass
               ...
               >>> def greet(name):
               ...     if name == 'Bob':
               ...         raise DontGreet(name)
               ...     return f"Hello, {name}!"
               ...
               >>> with_greet = Namespace(globals())
               >>> schema = Schema({
               ...    'given': str,
               ...    **with_greet.error_or({
               ...        'expected': str,
               ...    }),
               ... })
            
               >>> p1 = schema({
               ...         'given': 'Alice',
               ...         'expected': 'Hello, Alice!',
               ... })
               >>> with p1['error']:
               ...    assert greet(p1['given']) == p1['expected']
            
               >>> p2 = schema({
               ...         'given': 'Bob',
               ...         'error': {
               ...             'type': 'DontGreet',
               ...             'message': 'Bob',
               ...         },
               ... })
               >>> with p2['error']:
               ...    assert greet(p2['given']) == p2['expected']
            
            Note that the with block is the exact same for both assertions, even 
            though one tests for an error and the other doesn't.  This is what 
            makes `error_or()` useful for writing parametrized tests.
        """

        # With the API I'm using (accept a dictionary, return a dictionary), 
        # it's not actually possible to validate that only an error or the 
        # expected values are passed.  To do so requires some validator that 
        # runs on the whole data structure, which would have to be setup like 
        # so:
        #
        #     schema = Schema(
        #             All(
        #                 ...,
        #                 ns.error_or({
        #                     ...
        #                 }),
        #             )
        #     )
        #
        # I could simplify this a bit by wrapping `Schema`:
        #
        #     schema = ns.Schema({
        #                 ...,
        #             },
        #             error_or={
        #             },
        #     )
        #
        # Another problem with the current API is that optional expected keys 
        # need to be handled specially.  All of the expected keys need to be 
        # made optional, but I don't want to override the default value for any 
        # already-optional keys, because that will affect how they behave in 
        # non-error tests.
        # 
        # Despite all that, I still like this API.  The alternatives are too 
        # complex.  I think the best way to think about `error_or()` is that 
        # it's an intuitive shortcut for common use cases.  The user can always 
        # build their own schemas using `error()` if they need something more 
        # complicated.

        schema = {
                Optional('error', default='none'): self.error,
        }

        def check(schema):
            # This function basically just reimplements `Or`, but with a better 
            # error message.  The error message for `Or` is confusing because it 
            # only mentions the first option and not the second.  That's especially 
            # bad in this case, where the first option is basically an internal 
            # implementation detail.  This function changes the error message so 
            # that only the second option is mentioned, which is the least 
            # confusing in this case.
            # 
            # It's worth noting that the real problem is that voluptuous checks 
            # default values.  I can't imagine a case where this is actually 
            # useful. I might think about making a PR for that.

            def do_check(x):
                if isinstance(x, MagicMock):
                    return x
                return Schema(schema)(x)

            return do_check

        for k, v in expected.items():
            if k == 'error':
                raise ValueError("the key 'error' is reserved for use by error_or()")
            elif isinstance(k, str):
                schema[Optional(k, default=MagicMock)] = check(v)
            elif isinstance(k, Optional):
                schema[k] = v
            else:
                raise TypeError("'error_or()' only supports the following key types: str, voluptuous.Optional")

        return schema




import math
import vector
import parametrize_from_file
from pytest import approx
from voluptuous import Schema, Optional
from parametrize_from_file.voluptuous import Namespace

with_math = Namespace().all(math)
with_vec = with_math.copy().all(vector)

@parametrize_from_file(
        schema=Schema({
            'angle': with_math.eval,
            Optional('kwargs', default={}): with_math.eval,
            'expected': with_vec.eval,
        }),
)
def test_from_angle(angle, kwargs, expected):
    actual = vector.from_angle(angle, **kwargs)

    assert actual.x == approx(expected.x)
    assert actual.y == approx(expected.y)



#!/usr/bin/env python3

import pytest
from parametrize_from_file import Namespace

class MockError1(Exception):
    pass

class MockError1a(MockError1):
    pass

class MockError2(Exception):
    pass

class SharedParams:
    use_init = pytest.mark.parametrize(
            'args, kwargs, expected', [

                # One name:
                ([{'a': 1}], {}, {'a': 1}),
                (['a = 1'], {}, {'a': 1}),
                ([], {'a': 1}, {'a': 1}),

                # Two names:
                ([{'a': 1, 'b': 2}], {}, {'a': 1, 'b': 2}),
                ([{'a': 1}, {'b': 2}], {}, {'a': 1, 'b': 2}),
                ([{'a': 1}, 'b = 2'], {}, {'a': 1, 'b': 2}),
                ([{'a': 1}], {'b': 2}, {'a': 1, 'b': 2}),

                (['a, b = 1, 2'], {}, {'a': 1, 'b': 2}),
                (['a = 1', {'b': 2}], {}, {'a': 1, 'b': 2}),
                (['a = 1', 'b = 2'], {}, {'a': 1, 'b': 2}),
                (['a = 1'], {'b': 2}, {'a': 1, 'b': 2}),

                ([{'b': 2}], {'a': 1}, {'a': 1, 'b': 2}),
                (['b = 2'], {'a': 1}, {'a': 1, 'b': 2}),
                ([], {'a': 1, 'b': 2}, {'a': 1, 'b': 2}),

                # Order matters:
                ([{'a': 1}, {'a': 2}], {}, {'a': 2}),
                ([{'a': 1}, 'a = 2'], {}, {'a': 2}),
                ([{'a': 1}], {'a': 2}, {'a': 2}),

                (['a = 1', {'a': 2}], {}, {'a': 2}),
                (['a = 1', 'a = 2'], {}, {'a': 2}),
                (['a = 1'], {'a': 2}, {'a': 2}),
            ],
    )
    eval_err = pytest.mark.parametrize(
            'globals, args, kwargs, error', [
                # Name errors:
                ({'a': 1}, ['b + 1'], {}, NameError),
                ({'a': 1}, ['a + 1', 'b + 2'], {}, NameError),
                ({'a': 1}, [['b + 1']], {}, NameError),
                ({'a': 1}, [{'a': 'b + 1'}], {}, NameError),
                ({'a': 1}, [{'b': '1'}], {'eval_keys': True}, NameError),

                # Unhashable keys:
                ({'a': 1}, [{'[a]': '1'}], {'eval_keys': True}, TypeError),
                ({'a': 1}, [{'{a: 1}': '1'}], {'eval_keys': True}, TypeError),

                # Regular errors:
                ({}, ['1/0'], {}, ZeroDivisionError),
                ({}, ['[][1]'], {}, IndexError),
                ({}, ['{}["x"]'], {}, KeyError),

                # Not list/dict/str:
                ({'a': 1}, [1], {}, TypeError),
                ({'a': 1}, [('a + 1', 'a + 2')], {}, TypeError),
            ],
    )
    exec_err = pytest.mark.parametrize(
            'globals, src, error', [
                ({'a': 1}, 'b', NameError),
                ({'a': 1}, '1/0', ZeroDivisionError),
                ({'a': 1}, 1, TypeError),
            ],
    )
    exec_and_lookup_err = pytest.mark.parametrize(
            'globals, src, key, error', [
                ({'a': 1}, 'b', '', NameError),
                ({'a': 1}, '1/0', '', ZeroDivisionError),
                ({'a': 1}, 1, '', TypeError),
                ({'a': 1}, 'b = a + 1', 'c', KeyError),
                ({'a': 1}, 'b = a + 1', lambda d: d['c'], KeyError),
            ],
    )
    error_err = pytest.mark.parametrize(
            'globals, params, error', [
                ({'E': MockError1}, 'F', NameError),
                ({'E': MockError1}, '1/0', ZeroDivisionError),
                ({'E': MockError1}, 1, TypeError),
            ],
    )

@SharedParams.use_init
def test_init(args, kwargs, expected):
    ns = Namespace(*args, **kwargs)
    for k, v in expected.items():
        assert ns[k] == v

def test_repr():
    ns = Namespace(a=1)
    assert repr(ns) == "Namespace({'a': 1})"

@SharedParams.use_init
def test_use(args, kwargs, expected):
    ns = Namespace().use(*args, **kwargs)
    for k, v in expected.items():
        assert ns[k] == v

def test_use_method_chaining():
    ns = Namespace().use(a=1).use(b=2)
    assert ns == {'a': 1, 'b': 2}

def test_use_method_chaining_overwrite():
    ns = Namespace().use(a=1).use(a=2)
    assert ns == {'a': 2}

def test_all_1():
    import mock_module
    ns = Namespace().all(mock_module)
    assert ns == {'a': 1}

def test_all_2():
    import mock_module_all
    ns = Namespace().all(mock_module_all)
    assert ns == {'b': 2}

def test_all_method_chaining():
    import mock_module
    import mock_module_all
    ns = Namespace().all(mock_module).all(mock_module_all)
    assert ns == {'a': 1, 'b': 2}

def test_all_method_chaining_overwrite():
    import mock_module
    import mock_module_overwrite
    ns = Namespace().all(mock_module).all(mock_module_overwrite)
    assert ns == {'a': 2}

def test_copy():
    ns1 = Namespace(a=1)
    ns2 = ns1.copy().use(b=2)

    assert ns1 == {'a': 1}
    assert ns2 == {'a': 1, 'b': 2}

@pytest.mark.parametrize(
        'globals, args, kwargs, expected', [
            # Basic data types:
            ({'a': 1}, ['a + 1'], {}, 2),
            ({'a': 1}, ['a + 1', 'a + 2'], {}, [2, 3]),
            ({'a': 1}, [['a + 1']], {}, [2]),
            ({'a': 1}, [['a + 1', 'a + 2']], {}, [2, 3]),
            ({'a': 1}, [{'a': 'a + 1'}], {}, {'a': 2}),
            ({'a': 1}, [{'a': 'a + 1'}], {'eval_keys': True}, {1: 2}),
            ({'a': 1}, [{'a': 'a + 1', 'b': 'a + 2'}], {}, {'a': 2, 'b': 3}),

            # Recursion:
            ({'a': 1}, [[['a + 1']]], {}, [[2]]),
            ({'a': 1}, [[{'a': 'a + 1'}]], {}, [{'a': 2}]),
            ({'a': 1}, [{'a': ['a + 1']}], {}, {'a': [2]}),
            ({'a': 1}, [{'a': ['a + 1']}], {'eval_keys': True}, {1: [2]}),
            ({'a': 1}, [{'a': {'b': 'a + 1'}}], {}, {'a': {'b': 2}}),
            ({'a': 1}, [{'a': {'a+1': 'a+2'}}], {'eval_keys': True}, {1: {2: 3}}),
        ],
)
def test_eval(globals, args, kwargs, expected):
    ns = Namespace(globals)
    assert ns.eval(*args, **kwargs) == expected

@SharedParams.eval_err
def test_eval_err(globals, args, kwargs, error):
    ns = Namespace(globals)
    with pytest.raises(error):
        ns.eval(*args, **kwargs)

def test_exec():
    ns1 = Namespace(a=1)
    ns2 = ns1.exec('b = a + 1')
    ns3 = ns2.exec('c = b + 1')

    assert ns1 == {'a': 1}

    # Can't simply compare to a dictionary anymore, because `exec()` adds a 
    # bunch of builtin names to the namespace.
    assert ns2['a'] == 1
    assert ns2['b'] == 2
    assert 'c' not in ns2

    assert ns3['a'] == 1
    assert ns3['b'] == 2
    assert ns3['c'] == 3

@SharedParams.exec_err
def test_exec_err(globals, src, error):
    ns = Namespace(globals)
    with pytest.raises(error):
        ns.exec(src)

@pytest.mark.parametrize(
        'globals, src, key, expected', [
            ({'a': 1}, 'b = a + 1', 'b', 2),
            ({'a': 1}, 'b = a + 1', lambda d: d['b'], 2),
        ],
)
def test_exec_and_lookup(globals, src, key, expected):
    ns = Namespace(globals)
    assert ns.exec_and_lookup(key)(src) == expected

@SharedParams.exec_and_lookup_err
def test_exec_and_lookup_err(globals, src, key, error):
    ns = Namespace(globals)
    with pytest.raises(error):
        ns.exec_and_lookup(key)(src)

@pytest.mark.parametrize(
        'globals, params, expected_error, unexpected_error', [
            (
                {'E': MockError1},
                'none',
                None,
                MockError2(),
            ), (
                {'E': MockError1},
                'E',
                MockError1,
                MockError2(),
            ), (
                {'E': MockError1},
                {'type': 'E'},
                MockError1,
                MockError2(),
            ), (
                {'E': MockError1},
                {'type': 'E'},
                MockError1a,
                MockError2(),
            ), (
                {'E': MockError1a},
                {'type': 'E'},
                MockError1a,
                MockError1(),
            ), (
                {'E': MockError1},
                {'type': 'E', 'message': 'a'},
                MockError1('a'),
                MockError1('b'),
            ), (
                {'E': MockError1},
                {'type': 'E', 'message': r'\d{3}'},
                MockError1('404'),
                MockError1(r'\d{3}'),
            ),
        ],
)
def test_error(globals, params, expected_error, unexpected_error):
    ns = Namespace(globals)
    cm = ns.error(params)

    with cm:
        if expected_error is not None:
            raise expected_error

    with pytest.raises((unexpected_error.__class__, AssertionError)):
        with cm:
            raise unexpected_error
    
@SharedParams.error_err
def test_error_err(globals, params, error):
    ns = Namespace(globals)
    with pytest.raises(error):
        ns.error(params)

def test_error_repr():
    ns = Namespace(E=MockError1)
    cm = ns.error('E')
    assert repr(cm) == "<ExpectError type=<class 'test_namespace.MockError1'> messages=[]>"




#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['parametrize_from_file']

package_data = \
{'': ['*']}

install_requires = \
['toml',
 'pyyaml',
 'nestedtext',
 'tidyexc',
 'pytest',
 'more_itertools',
 'contextlib2']

extras_require = \
{'doc': ['sphinx', 'sphinx_rtd_theme', 'autoclasstoc', 'sphinx-toolbox'],
 'test': ['pytest', 'pytest-cov', 'coveralls', 'voluptuous', 'numpy']}

setup(name='parametrize_from_file',
      version='0.5.1',
      description='Parametrize test functions with values read from config files.',
      author='Kale Kundert',
      author_email='kale@thekunderts.net',
      url='https://github.com/kalekundert/parametrize_from_file',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='~=3.6',
     )
